"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureInstalled = void 0;
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
async function ensureInstalled(options) {
    if (!isInstallRequired(options)) {
        return;
    }
    await installRequiredPackages(options);
}
exports.ensureInstalled = ensureInstalled;
function isInstallRequired(options) {
    const versionCachePath = installedVersionInfoPath(options);
    return (!(0, fs_1.existsSync)(versionCachePath) ||
        (0, fs_1.readFileSync)(versionCachePath, "utf8") !== versionId(options));
}
const MAX_WAIT_SECONDS = 60;
async function installRequiredPackages(options) {
    options.status.info("Please wait while Preview.js installs dependencies…");
    const singleInstallLockPack = path_1.default.join(options.installDir, "__install_lock__");
    const packageJsonPath = path_1.default.join(options.installDir, "package.json");
    const expectedPackageLockPath = path_1.default.join(options.installDir, "package-lock.json");
    if ((0, fs_1.existsSync)(singleInstallLockPack)) {
        // Wait until installation is successful in another process.
        const waitingSince = Date.now();
        while (!(0, fs_1.existsSync)(expectedPackageLockPath)) {
            if (Date.now() - waitingSince > MAX_WAIT_SECONDS * 1000) {
                (0, fs_1.rmSync)(singleInstallLockPack);
                options.status.error(`Uh-oh. Unable to install Preview.js dependencies.`);
                throw new Error("Waiting too long for packages to be installed.");
            }
            await new Promise((resolve) => setTimeout(resolve, 1000));
        }
        options.status.info("Preview.js is ready.");
        return;
    }
    (0, fs_1.mkdirSync)(options.installDir, { recursive: true });
    (0, fs_1.writeFileSync)(singleInstallLockPack, Date.now().toString(), "utf8");
    process.on("exit", () => {
        (0, fs_1.unlinkSync)(singleInstallLockPack);
    });
    (0, fs_1.writeFileSync)(packageJsonPath, JSON.stringify({
        dependencies: {
            [options.packageName]: options.packageVersion,
        },
    }));
    return new Promise((resolve, reject) => {
        (0, child_process_1.exec)(`npm install`, {
            cwd: options.installDir,
        }, (error, stdout, stderr) => {
            console.log(stdout);
            console.error(stderr);
            if (error) {
                options.status.error(`Preview.js could not install dependencies:\n${error}`);
                return reject(error);
            }
            (0, fs_1.writeFileSync)(installedVersionInfoPath(options), versionId(options), "utf8");
            options.status.info("Preview.js is ready!");
            return resolve();
        });
    });
}
function installedVersionInfoPath(options) {
    return path_1.default.join(options.installDir, ".install.info");
}
function versionId(options) {
    return `${options.packageName}-${options.packageVersion}`;
}
